#import "Plugin.h"
#import <Foundation/Foundation.h>
#import <Capacitor/Capacitor-Swift.h>
#import <AVFoundation/AVFoundation.h>
#import <CoreBluetooth/CoreBluetooth.h>
#import <CoreLocation/CoreLocation.h>

@interface UcleanPermissionPlugin()<CBCentralManagerDelegate>

@property(nonatomic,strong) CBCentralManager *centralManager;
@property(nonatomic,strong) NSMutableDictionary *permissionDic;
@property(nonatomic,strong) NSArray *permissionArray;
@property(nonatomic,strong) CAPPluginCall *permissionCall;

@end

@implementation UcleanPermissionPlugin


CAP_PLUGIN_CONFIG(UcleanPermissionPlugin, "UcleanPermissionPlugin");

+(CAPPluginMethod *)getMethod:(NSString *)methodName {
    NSArray *methods = [self pluginMethods];
    for (CAPPluginMethod *method in methods) {
        if ([method.name isEqualToString:methodName]) {
            return method;
        }
    }
    return nil;
}

//添加插件相关方法
+(NSArray *)pluginMethods {
    NSMutableArray *methods = [NSMutableArray new];
    CAP_PLUGIN_METHOD(echo, CAPPluginReturnPromise);
    CAP_PLUGIN_METHOD(checkSelfPermissions, CAPPluginReturnPromise);
    CAP_PLUGIN_METHOD(setPermissions, CAPPluginReturnPromise);
    CAP_PLUGIN_METHOD(jumpToSystemSetting, CAPPluginReturnPromise);
    return methods;
}

#pragma mark - 插件方法实现
- (void)echo:(CAPPluginCall *)call {
    NSDictionary *paramDic = call.options;
    NSString *value = paramDic[@"value"];
    CAPPluginCallResult *callResult = [[CAPPluginCallResult alloc] init:@{@"value":value}];
    call.successHandler(callResult, call);
}

- (void)checkSelfPermissions:(CAPPluginCall *)call {
    self.permissionCall = call;
    NSDictionary *paramDic = call.options;
    NSArray *permissions = paramDic[@"permissions"];
    if (!permissions || ![permissions isKindOfClass:NSArray.class] || permissions.count == 0) {
        CAPPluginCallResult *callResult = [[CAPPluginCallResult alloc] init:@{@"errorCode":@"-1",@"errorMessage":@"参数错误"}];
        call.successHandler(callResult, call);
        return;
    }
    
    [self.permissionDic removeAllObjects];
    
    if (permissions.count == 1 && [permissions containsObject:@"storage"]) {
        //仅仅检测存储权限
        CAPPluginCallResult *callResult = [[CAPPluginCallResult alloc] init:@{@"errorCode":@"0",
                                                                              @"errorMessage":@"",
                                                                                         @"data":@[@{@"storage":@(YES)}]}];
        call.successHandler(callResult, call);
        return;
    }
    
    self.permissionArray = permissions;
    for (int i = 0; i < permissions.count; i ++) {
        NSString *authstr = permissions[i];
        if ([authstr containsString:@"storage"]) {
            self.permissionDic[@"storage"] = @(YES);
            
        }else if ([authstr containsString:@"camera"]) {
            [self detectionCameraState];
            
        } else if ([authstr containsString:@"bluetoothle"]) {
            [self detectionBluetoothleState];
            
        } else if ([authstr containsString:@"location"])
        {
            [self detectionLocationState];
        }
    }
}

- (void)setPermissions:(CAPPluginCall *)call {
    CAPPluginCallResult *callResult = [[CAPPluginCallResult alloc] init:@{@"errorCode":@"-1",@"errorMessage":@"iOS需要用户手动设置权限"}];
    call.successHandler(callResult, call);
}

- (void)jumpToSystemSetting:(CAPPluginCall *)call {
    NSURL * url = [NSURL URLWithString:UIApplicationOpenSettingsURLString];
    if ([[UIApplication sharedApplication] canOpenURL:url]) {
        if (@available(iOS 10.0, *)) {
            [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
        } else {
            [[UIApplication sharedApplication] openURL:url];
            
        }
    }
    
    CAPPluginCallResult *callResult = [[CAPPluginCallResult alloc] init:@{@"errorCode":@"0",@"errorMessage":@"",}];
    call.successHandler(callResult, call);
}

- (void)checkPermissionIsFinish {
    //检测权限是否完成
    NSMutableArray *array = [[NSMutableArray alloc] init];
    //是否检测完成
    BOOL isAuthFinish = YES;
    for (NSString *permission in self.permissionArray) {
        if ([self.permissionDic valueForKey:permission]) {
            [array addObject:@{permission:[self.permissionDic valueForKey:permission]}];
        } else {
            isAuthFinish = NO;
            break;
        }
    }
    
    if (isAuthFinish) {
        
        CAPPluginCallResult *callResult = [[CAPPluginCallResult alloc] init:@{@"errorCode":@"0",
                                                                              @"errorMessage":@"",
                                                                              @"data":array}];
        if (self.permissionCall) {
            self.permissionCall.successHandler(callResult, self.permissionCall);
        }
    }
}

///检测相机访问权限
-(void)detectionCameraState
{
    if (self.permissionDic[@"camera"]) {
        return;
    }
    NSString *mediaType = AVMediaTypeVideo;//读取媒体类型
    [AVCaptureDevice requestAccessForMediaType:mediaType completionHandler:^(BOOL granted) {
        self.permissionDic[@"camera"] = @(granted);
        [self checkPermissionIsFinish];
    }];
    
}
///检测定位访问权限
-(void)detectionLocationState
{
    if (self.permissionDic[@"location"]) {
        return;
    }
    if ([CLLocationManager locationServicesEnabled] && ([CLLocationManager authorizationStatus] == kCLAuthorizationStatusAuthorizedWhenInUse || [CLLocationManager authorizationStatus] == kCLAuthorizationStatusNotDetermined || [CLLocationManager authorizationStatus] ==kCLAuthorizationStatusAuthorizedAlways))
    {
        //定位功能可用
        self.permissionDic[@"location"] = @(YES);
        self.permissionDic[@"locationService"] = @(YES);
    } else {
        
        //定位不能用
        self.permissionDic[@"location"] = @(NO);
        self.permissionDic[@"locationService"] = @(NO);
    }
    
    [self checkPermissionIsFinish];
}

- (void)detectionBluetoothleState
{
    if (self.permissionDic[@"bluetoothle"]) {
        return;
    }
    self.centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:nil options:nil];
}

- (void)centralManagerDidUpdateState:(CBCentralManager *)central {
    BOOL kgstate = NO;
    BOOL authstate = NO;
    if (central.state == CBManagerStatePoweredOn) {
        kgstate = YES;
        authstate = YES;
    }
    
    if (central.state == CBManagerStatePoweredOff) {
        kgstate = NO;
        authstate = YES;
    }
    
    if (central.state == CBManagerStateUnauthorized) {
        kgstate = NO;
        authstate = NO;
    }
    
    self.permissionDic[@"bluetoothle"] = @(authstate);
    self.permissionDic[@"bluetoothleOpen"] = @(kgstate);
    [self checkPermissionIsFinish];
}

- (NSMutableDictionary *)permissionDic {
    if (!_permissionDic) {
        _permissionDic = [[NSMutableDictionary alloc] init];
    }
    return _permissionDic;
}

@end
